@@ -2,7 +2,7 @@
  * @file decode.c
  *
  * @section LICENSE
- * Copyright 2012 - 2015
+ * Copyright 2012 - 2016
  *
  * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
  * Deutsches Klimarechenzentrum GmbH
@@ -60,7 +60,7 @@
 #endif
 
 #define ROS 5
-
+#define RSI_USED(state) ((size_t)(state->rsip - state->rsi_buffer))
 #define BUFFERSPACE(strm) (strm->avail_in >= strm->state->in_blklen      \
                            && strm->avail_out >= strm->state->out_blklen)
 
@@ -203,7 +203,7 @@ static inline void check_rsi_end(struct aec_stream *strm)
      */
     struct internal_state *state = strm->state;
 
-    if (state->rsi_size == (size_t)(state->rsip - state->rsi_buffer)) {
+    if (state->rsi_size == RSI_USED(state)) {
         state->flush_output(strm);
         state->flush_start = state->rsi_buffer;
         state->rsip = state->rsi_buffer;
@@ -388,7 +388,7 @@ static inline void fs_drop(struct aec_stream *strm)
 static inline uint32_t copysample(struct aec_stream *strm)
 {
     if (bits_ask(strm, strm->bits_per_sample) == 0
-        || strm->avail_out == 0)
+        || strm->avail_out < strm->state->bytes_per_sample)
         return 0;
 
     put_sample(strm, bits_get(strm, strm->bits_per_sample));
@@ -423,7 +423,7 @@ static int m_split_output(struct aec_stream *strm)
     int k = state->id - 1;
 
     do {
-        if (bits_ask(strm, k) == 0 || strm->avail_out == 0)
+        if (bits_ask(strm, k) == 0 || strm->avail_out < state->bytes_per_sample)
             return M_EXIT;
         if (k)
             *state->rsip++ += bits_get(strm, k);
@@ -458,7 +458,8 @@ static int m_split_fs(struct aec_stream *strm)
 
 static int m_split(struct aec_stream *strm)
 {
-    uint32_t i, k;
+    size_t i;
+    int k;
     struct internal_state *state = strm->state;
 
     if (BUFFERSPACE(strm)) {
@@ -502,7 +503,7 @@ static int m_zero_output(struct aec_stream *strm)
     struct internal_state *state = strm->state;
 
     do {
-        if (strm->avail_out == 0)
+        if (strm->avail_out < state->bytes_per_sample)
             return M_EXIT;
         put_sample(strm, 0);
     } while(--state->i);
@@ -522,7 +523,7 @@ static int m_zero_block(struct aec_stream *strm)
     fs_drop(strm);
 
     if (zero_blocks == ROS) {
-        b = (int)(state->rsip - state->rsi_buffer) / strm->block_size;
+        b = (int)RSI_USED(state) / strm->block_size;
         zero_blocks = MIN(strm->rsi - b, 64 - (b % 64));
     } else if (zero_blocks > ROS) {
         zero_blocks--;
@@ -536,7 +537,7 @@ static int m_zero_block(struct aec_stream *strm)
     zero_bytes = i * state->bytes_per_sample;
 
     if (strm->avail_out >= zero_bytes) {
-        if (state->rsi_size - (state->rsip - state->rsi_buffer) < i)
+        if (state->rsi_size - RSI_USED(state) < i)
             return M_ERROR;
 
         memset(state->rsip, 0, i * sizeof(uint32_t));
@@ -565,13 +566,13 @@ static int m_se_decode(struct aec_stream *strm)
         d1 = m - state->se_table[2 * m + 1];
 
         if ((state->i & 1) == 0) {
-            if (strm->avail_out == 0)
+            if (strm->avail_out < state->bytes_per_sample)
                 return M_EXIT;
             put_sample(strm, state->se_table[2 * m] - d1);
             state->i++;
         }
 
-        if (strm->avail_out == 0)
+        if (strm->avail_out < state->bytes_per_sample)
             return M_EXIT;
         put_sample(strm, d1);
         state->i++;
@@ -654,7 +655,7 @@ static int m_uncomp_copy(struct aec_stream *strm)
 
 static int m_uncomp(struct aec_stream *strm)
 {
-    uint32_t i;
+    size_t i;
     struct internal_state *state = strm->state;
 
     if (BUFFERSPACE(strm)) {
@@ -814,6 +815,10 @@ int aec_decode(struct aec_stream *strm, int flush)
     if (status == M_ERROR)
         return AEC_DATA_ERROR;
 
+    if (status == M_EXIT && strm->avail_out > 0 &&
+        strm->avail_out < state->bytes_per_sample)
+        return AEC_MEM_ERROR;
+
     state->flush_output(strm);
 
     strm->total_in -= strm->avail_in;
