@@ -2,7 +2,7 @@
  * @file encode.c
  *
  * @section LICENSE
- * Copyright 2012 - 2015
+ * Copyright 2012 - 2016
  *
  * Mathis Rosenhauer, Moritz Hanke, Joerg Behrens
  * Deutsches Klimarechenzentrum GmbH
@@ -117,7 +117,7 @@ static inline void copy64(uint8_t *dst, uint64_t src)
 
 static inline void emitblock_fs(struct aec_stream *strm, int k, int ref)
 {
-    uint32_t i;
+    size_t i;
     uint32_t used; /* used bits in 64 bit accumulator */
     uint64_t acc; /* accumulator */
     struct internal_state *state = strm->state;
@@ -247,7 +247,7 @@ static void preprocess_unsigned(struct aec_stream *strm)
     uint32_t *restrict d = state->data_pp;
     uint32_t xmax = state->xmax;
     uint32_t rsi = strm->rsi * strm->block_size - 1;
-    unsigned int i;
+    size_t i;
 
     state->ref = 1;
     state->ref_sample = x[0];
@@ -284,7 +284,7 @@ static void preprocess_signed(struct aec_stream *strm)
     int32_t xmin = (int32_t)state->xmin;
     uint32_t rsi = strm->rsi * strm->block_size - 1;
     uint32_t m = UINT64_C(1) << (strm->bits_per_sample - 1);
-    unsigned int i;
+    size_t i;
 
     state->ref = 1;
     state->ref_sample = x[0];
@@ -316,7 +316,7 @@ static inline uint64_t block_fs(struct aec_stream *strm, int k)
        Sum FS of all samples in block for given splitting position.
     */
 
-    uint32_t i;
+    size_t i;
     uint64_t fs = 0;
     struct internal_state *state = strm->state;
 
@@ -417,7 +417,7 @@ static uint32_t assess_se_option(struct aec_stream *strm)
        If length is above limit just return UINT32_MAX.
     */
 
-    uint32_t i;
+    size_t i;
     uint64_t len, d;
     struct internal_state *state = strm->state;
     uint32_t *block = state->block;
@@ -546,7 +546,7 @@ static int m_encode_uncomp(struct aec_stream *strm)
 
 static int m_encode_se(struct aec_stream *strm)
 {
-    uint32_t i;
+    size_t i;
     uint32_t d;
     struct internal_state *state = strm->state;
 
@@ -620,7 +620,7 @@ static int m_check_zero_block(struct aec_stream *strm)
        end of a segment or RSI.
     */
 
-    uint32_t i;
+    size_t i;
     struct internal_state *state = strm->state;
     uint32_t *p = state->block;
 
@@ -646,10 +646,10 @@ static int m_check_zero_block(struct aec_stream *strm)
             state->zero_ref = state->ref;
             state->zero_ref_sample = state->ref_sample;
         }
-        if (state->blocks_avail == 0
-            || (strm->rsi - state->blocks_avail) % 64 == 0) {
+        if (state->blocks_avail == 0 || state->blocks_dispensed % 64 == 0) {
             if (state->zero_blocks > 4)
                 state->zero_blocks = ROS;
+
             state->mode = m_encode_zero;
             return M_CONTINUE;
         }
@@ -728,6 +728,7 @@ static int m_get_block(struct aec_stream *strm)
     if (state->blocks_avail == 0) {
         state->blocks_avail = strm->rsi - 1;
         state->block = state->data_pp;
+        state->blocks_dispensed = 1;
 
         if (strm->avail_in >= state->rsi_len) {
             state->get_rsi(strm);
@@ -745,6 +746,7 @@ static int m_get_block(struct aec_stream *strm)
             state->uncomp_len = strm->block_size * strm->bits_per_sample;
         }
         state->block += strm->block_size;
+        state->blocks_dispensed++;
         state->blocks_avail--;
         return m_check_zero_block(strm);
     }
